'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = generate;

var _color = require('color');

var _color2 = _interopRequireDefault(_color);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var hueStep = 2; // 色相阶梯
var saturationStep = 16; // 饱和度阶梯，浅色部分
var saturationStep2 = 5; // 饱和度阶梯，深色部分
var brightnessStep1 = 5; // 亮度阶梯，浅色部分
var brightnessStep2 = 15; // 亮度阶梯，深色部分
var lightColorCount = 5; // 浅色数量，主色上
var darkColorCount = 4; // 深色数量，主色下

function getHue(hsv, i, light) {
  var hue = void 0;
  // 根据色相不同，色相转向不同
  if (hsv.color[0] >= 60 && hsv.color[0] <= 240) {
    hue = light ? hsv.color[0] - hueStep * i : hsv.color[0] + hueStep * i;
  } else {
    hue = light ? hsv.color[0] + hueStep * i : hsv.color[0] - hueStep * i;
  }
  if (hue < 0) {
    hue += 360;
  } else if (hue >= 360) {
    hue -= 360;
  }
  return hue;
}

function getSaturation(hsv, i, light) {
  var saturation = void 0;
  if (light) {
    saturation = hsv.color[1] - saturationStep * i;
  } else if (i === darkColorCount) {
    saturation = hsv.color[1] + saturationStep;
  } else {
    saturation = hsv.color[1] + saturationStep2 * i;
  }
  // 边界值修正
  if (saturation > 100) {
    saturation = 100;
  }
  // 第一格的 s 限制在 6-10 之间
  if (light && i === lightColorCount && saturation > 10) {
    saturation = 10;
  }
  if (saturation < 6) {
    saturation = 6;
  }
  return saturation;
}

function getValue(hsv, i, light) {
  if (light) {
    return hsv.color[2] + brightnessStep1 * i;
  }
  return hsv.color[2] - brightnessStep2 * i;
}

function generate(color) {
  var patterns = [];
  var pColor = (0, _color2.default)(color);
  for (var i = lightColorCount; i > 0; i -= 1) {
    var hsv = pColor.hsv().round();
    var colorString = (0, _color2.default)({
      h: getHue(hsv, i, true),
      s: getSaturation(hsv, i, true),
      v: getValue(hsv, i, true)
    }).hex();
    patterns.push(colorString);
  }
  patterns.push(pColor.hex());
  for (var _i = 1; _i <= darkColorCount; _i += 1) {
    var _hsv = pColor.hsv().round();
    var _colorString = (0, _color2.default)({
      h: getHue(_hsv, _i),
      s: getSaturation(_hsv, _i),
      v: getValue(_hsv, _i)
    }).hex();
    patterns.push(_colorString);
  }
  return patterns;
}